from distutils.core import setup

setup(name='django-profiles2',
      version='0.6',
      description='User-profile application for Django',
      author='James Bennett',
      author_email='james@b-list.org',
      maintainer='flavio.curella@gmail.com',
      url='http://www.bitbucket.org/fcurella/django-profiles/wiki/',
      download_url='http://www.bitbucket.org/fcurella/django-profiles/get/v0.5.gz',
      packages=['profiles'],
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Utilities'],
      )